package cn.fw.dalaran.service.data.impl;

import cn.fw.dalaran.dao.VideoPoolDao;
import cn.fw.dalaran.domain.db.Account;
import cn.fw.dalaran.domain.db.ActivityTheme;
import cn.fw.dalaran.domain.db.VideoPool;
import cn.fw.dalaran.domain.dto.PoolQueryDTO;
import cn.fw.dalaran.domain.vo.VideoPoolVO;
import cn.fw.dalaran.service.data.AccountService;
import cn.fw.dalaran.service.data.ActivityThemeService;
import cn.fw.dalaran.service.data.VideoPoolService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @date 2021-11-17 14:54
 * @description VideoPoolService
 */
@Service
@RequiredArgsConstructor
@SuppressWarnings("Duplicates")
public class VideoPoolServiceImpl extends ServiceImpl<VideoPoolDao, VideoPool> implements VideoPoolService {

    private final ActivityThemeService activityThemeService;
    private final AccountService accountService;

    @Override
    public void removeAccountDataByDate(Long accountId, LocalDate date) {
        this.getBaseMapper().deleteByAccountDate(accountId, date);
    }

    @Override
    public List<VideoPoolVO> videoList(PoolQueryDTO queryDTO) {
        int current = queryDTO.getCurrent() <= 0 ? 1 : queryDTO.getCurrent();
        Integer pageSize = queryDTO.getPageSize();
        Integer startIndex = (current - 1) * pageSize;
        return Optional.ofNullable(getBaseMapper().videoList(startIndex, pageSize, queryDTO)).orElse(new ArrayList<>());
    }

    @Override
    public long videoCount(PoolQueryDTO queryDTO) {
        return Optional.ofNullable(getBaseMapper().videoCount(queryDTO)).orElse(0L);
    }

    /**
     * 获取主题活动员工有效视频数
     *
     * @param userId  用户id
     * @param themeId 活动主题id
     * @return 有效视频数
     */
    @Override
    public Integer getValidVideoCnt(Long userId, Long themeId) {
        final ActivityTheme activityTheme = activityThemeService.getById(themeId);
        if (Objects.isNull(activityTheme))
            return 0;
        final Set<Long> accountIds = accountService.lambdaQuery()
                .eq(Account::getUserId, userId)
                .list()
                .stream()
                .map(Account::getId)
                .collect(Collectors.toSet());
        if (Objects.equals(accountIds.size(), 0))
            return 0;
        final List<VideoPool> list = this.lambdaQuery()
                .in(VideoPool::getAccountId, accountIds)
                .eq(VideoPool::getValidVideo, 1)
                .ge(VideoPool::getPublishTime, activityTheme.getStartTime())
                .le(VideoPool::getPublishTime, activityTheme.getEndTime())
                .list();
        if (Objects.equals(list.size(), 0))
            return 0;
        return list.stream().collect(Collectors.groupingBy(VideoPool::getVideoId)).size();
    }

    /**
     * 找到最新的video信息
     *
     * @param videoId 视频id
     * @return 该视频id对应的最新的视频
     */
    @Override
    public VideoPool getLatestVideo(String videoId) {
        final List<VideoPool> videoList = this.lambdaQuery()
                .eq(VideoPool::getVideoId, videoId)
                .orderByDesc(VideoPool::getId)
                .list();
        if (videoList.size() > 0) {
            return videoList.get(0);
        }
        return null;
    }
}
